/*
   grid2d.c
   San Skulrattanakulchai
   11/11/99
 */
#include <stdlib.h>   /* for NULL */
#include <assert.h>   /* for assert() */
#include "digraph.h"
#include "intarray.h"
#include "gen.h"
#include "random.h"
#include "util.h"

#define PERMUTE_NAMES \
{\
  makeIntArray(size, a);\
  for (j = 1; j <= size; j++)\
    a[j] = j;\
  for (j = size; j >= 2; j--)\
    if ((k = randNum(1, j)) != j) {\
      temp = a[j]; a[j] = a[k]; a[k] = temp;\
    }\
}

#define ADD_EDGES_BETWEEN_v_AND_w	\
{\
  if (!oriented && !pmtName) {\
    if (coinFlip())\
      addEdge(v, w, Gp);\
    if (coinFlip())\
      addEdge(w, v, Gp);\
  }\
  if (!oriented && pmtName) {\
    if (coinFlip())\
      addEdge(a[v], a[w], Gp);\
    if (coinFlip())\
      addEdge(a[w], a[v], Gp);\
  }\
  if (oriented && !pmtName) {\
    if (coinFlip()) {\
      addEdge(v, w, Gp);\
    } else {\
      addEdge(w, v, Gp);\
    }\
  }\
  if (oriented && pmtName) {\
    if (coinFlip()) {\
      addEdge(a[v], a[w], Gp);\
    } else {\
      addEdge(a[w], a[v], Gp);\
    }\
  }\
}

/*
   Generate a random r x s rectangular grid.
 */
DigraphPtr grid2d(int r, int s, bool wraparound, bool oriented, bool pmtName)
{
  int size, x, y, x1, y1, temp, j, k;
  DigraphPtr Gp;
  Vertex v, w;
  int *a = NULL;

  if (wraparound)
    assert(r >= 3 && s >= 3);

  size = r*s; makeEmptyDigraph(size, Gp);

  if (pmtName)
    PERMUTE_NAMES

  for (y = 1; y <= s; y++)
    for (x = 1; x <= r; x++) {
      v = (y-1)*r + x;
      if (x < r || (x == r && wraparound)) {
        x1 = (x==r) ? 1 : x+1;
        w = (y-1)*r + x1;
        ADD_EDGES_BETWEEN_v_AND_w
      }
      if (y < s || (y == s && wraparound)) {
        y1 = (y==s) ? 1 : y+1;
        w = (y1-1)*r + x;
        ADD_EDGES_BETWEEN_v_AND_w
      }
    }

  if (pmtName)
    deleteIntArray(a);

  return Gp;
}
