/*
   gscc.C
   San Skulrattanakulchai
   11/11/99
   Implementation of Gabow's strong component algorithm using path-based dfs.
 */
#include "digraph.h"
#include "gscc.h"
#include "intarray.h"
#include "stack.h"

static int gc;  /* component number */
static Stack S;
static Stack B;

int gstrong(DigraphPtr Gp, IntArray I)
{
  Vertex v;

  makeStack(Gp->n, S); makeStack(Gp->n, B);

  gc = Gp->n;
  for (v = 1; v <= Gp->n; v++)
    if (I[v] == 0)
      gdfs(v, Gp, I);

  deleteStack(B); deleteStack(S);

  return gc - Gp->n;
}

void gdfs(Vertex v, DigraphPtr Gp, IntArray I)
{
  NodePtr p;
  Vertex w;

  push(v, S); I[v] = top(S); push(I[v], B);  /* add v to the end of P */

  for (p = Gp->adj[v]; p; p = p->next)
    if (I[w = p->vertex] == 0)
      gdfs(w, Gp, I);
    else  /* contract vertices */
      while (B[top(B)] > I[w])
	voidPop(B);

  if (B[top(B)] == I[v]) {
    /* number vertices of the next strong component */
    voidPop(B);
    gc++;
    while (top(S) >= I[v])
      I[pop(S)] = gc;
  }
}
