/*
   kscc.C
   San Skulrattanakulchai
   11/11/99
   Implemnentation of Kosaraju & Sharir's 2-pass strong component algorithm.
 */
#include "digraph.h"
#include "intarray.h"
#include "kscc.h"

static int ftime;            /* finish time */
static int kc;               /* component number */
static IntArray discovered;  /* 0-1 array, its meaning depends on the pass */
static IntArray finish;      /* finish[i] = v implies finish time of v is i */

int kstrong(DigraphPtr Gp, IntArray K)
{
  Vertex v;
  int nv = Gp->n, i;

  makeIntArray(nv, discovered);
  makeIntArray(nv, finish);

  ftime = 0;
  for (v = 1; v <= nv; v++)
    if (discovered[v] == 0)  /* v not yet discovered in 1st pass */
      pass1DFS(v, Gp);

  kc = nv;
  for (i = nv; i >= 1; i--)
    if (K[v = finish[i]] == 0) {  /* v hasn't been assigned a component # */
      ++kc;
      pass2DFS(v, Gp, K);
    }

  deleteIntArray(finish);
  deleteIntArray(discovered);

  return kc - nv;
}

void pass1DFS(Vertex v, DigraphPtr Gp)
{
  NodePtr p;
  Vertex w;

  discovered[v] = 1;  /* mark v discovered in 1st pass */

  for (p = Gp->adj[v]; p; p = p->next)
    if (discovered[w = p->vertex] == 0)  /* w not yet discovered in 1st pass */
      pass1DFS(w, Gp);

  finish[++ftime] = v;
}

void pass2DFS(Vertex v, DigraphPtr Gp, IntArray K)
{
  NodePtr np;
  Vertex w;

  discovered[v] = 0;  /* mark v discovered in 2nd pass */
  K[v] = kc;

  for (np = Gp->radj[v]; np; np = np->next)
    if (discovered[w = np->vertex] == 1)  /* w not yet discovered in 2nd pass */
      pass2DFS(w, Gp, K);
}
