/*
   kscc1.C
   San Skulrattanakulchai
   11/11/99
   Implemnentation of Kosaraju & Sharir's 2-pass strong component algorithm.
   Use K[] with different meanings.
 */
#include "digraph.h"
#include "intarray.h"
#include "kscc.h"

static int ftime;            /* finish time */
static int kc;               /* component number */
static IntArray finish;      /* finish[i] = v implies finish time of v is i */

int kstrong1(DigraphPtr Gp, IntArray K)
{
  Vertex v;
  int nv = Gp->n, i;

  makeIntArray(Gp->n, finish);

  ftime = 0;
  for (v = 1; v <= nv; v++)
    if (K[v] == 0)  /* v not yet discovered in 1st pass */
      pass1DFS1(v, Gp, K);

  for (i = kc = nv; i >= 1; i--)
    if (K[v = finish[i]] == 1) {  /* v hasn't been assigned a component # */
      ++kc;
      pass2DFS1(v, Gp, K);
    }

  deleteIntArray(finish);

  return kc - nv;
}

void pass1DFS1(Vertex v, DigraphPtr Gp, IntArray K)
{
  NodePtr p;
  Vertex w;

  K[v] = 1;  /* mark v discovered in 1st pass */
  for (p = Gp->adj[v]; p; p = p->next)
    if (K[w = p->vertex] == 0)  /* w not yet discovered in 1st pass */
      pass1DFS1(w, Gp, K);

  finish[++ftime] = v;
}

void pass2DFS1(Vertex v, DigraphPtr Gp, IntArray K)
{
  NodePtr np;
  Vertex w;

  K[v] = kc;
  for (np = Gp->radj[v]; np; np = np->next)
    if (K[w = np->vertex] == 1)  /* w not yet discovered in 2nd pass */
      pass2DFS1(w, Gp, K);
}
